@@ -48,6 +48,9 @@ if 1 { # Not working yet?
             # Assume lowercase-convention for public commands
             set exports [list {[a-z]*}]
         }
+        if {[info exists ::syntaxbuild_allnamespace]} {
+            set exports [list *]
+        }
         foreach pat $exports {
             foreach p [info commands ${ns}::$pat] {
                 # Do not include the first :: in the name
@@ -158,15 +161,16 @@ proc buildDb {ch} {
     # o  Option, i.e anything starting with -
     # p  Option+Any (p as in option Pair)
     # s  Subcommand
-    # d  Definition. That arg defines a new command
-    #    If followed by =cmd, it copies syntax from cmd.
+    # d* Definitions. That arg defines a new command, as detailed below
     # di Define inheritance
-    # dc Define with copy
-    # do Define object
+    # dc Define with copy, if followed by =cmd, it copies syntax from cmd.
+    # do Define object, if followed by =cmd, it copies syntax from cmd.
     # dk Define constructor (args+body)
+    # dd Define destructor (just body)
     # dp Define procedure (name+args+body)
     # dm Define method (name+args+body)
     # dmp Define metod/procedure (name+args+body)
+    # div Define implicit variable
     # e  Expression
     # E  Expression that should be in braces
     # c  Code, checked in surrounding context
@@ -289,26 +293,20 @@ proc buildDb {ch} {
     set syntax(interp\ invokehidden) "x o* x x*"
     set syntax(join)            "r 1 2"
     set syntax(lappend)         "n x*"
-    if {[catch {lindex apa 0 0}]} {
-        set syntax(lindex)       2        ;# Pre 8.4
-    } else {
-        set syntax(lindex)      "r 2"
-    }
+    set syntax(lindex)          "r 1"
     set syntax(linsert)         "r 3"
     set syntax(list)            "r 0"
     set syntax(llength)          1
     set syntax(load)            "r 1 3"
     set syntax(lrange)           3
     set syntax(lreplace)        "r 3"
-    if {[catch {lsearch -all -glob apa bepa}]} {
-        set syntax(lsearch)     "o? x x"  ;# Pre 8.4
-    } else {
-        set syntax(lsearch)     "o* x x"
-    }
+    set syntax(lsearch)         "o* x x"
     set syntax(lset)            "n x x x*"
     set syntax(lsort)           "o* x"
     # "namespace" is handled specially
     set syntax(namespace)       "s x*" ;# FIXA: All subcommands
+    set syntax(namespace\ code) "c"
+    set return(namespace\ code) "script"
     set syntax(namespace\ import) "o* x*"
     set syntax(namespace\ which) "o* x?"
     set option(namespace\ which) "-variable -command"
@@ -357,7 +355,9 @@ proc buildDb {ch} {
     set syntax(string\ wordend)   2
     set syntax(string\ wordstart) 2
     set syntax(subst)           "o* x"
+    # "switch" is handled specially, but is added here to not disturb header gen
     set special(switch) 1
+    set syntax(switch)          "x*"
     set syntax(tell)             1
     set syntax(time)            "c x?"
     set syntax(trace)           "s x x*"
@@ -390,7 +390,8 @@ proc buildDb {ch} {
 
     # Things added in 8.5
     if {[info commands dict] ne ""} {
-        set syntax(dict)          "s x*"
+        set syntax(catch)        "c n? n?" ;# FIXA make a test for this
+        set syntax(dict)          "s x x*"
         set syntax(dict\ append)  "n x x*"
         set syntax(dict\ incr)    "n x x*"
         set syntax(dict\ filter)  "x x x"
@@ -430,12 +431,13 @@ proc buildDb {ch} {
         set syntax(apply)      "x x*"
         set syntax(source)     "p* x"
         set option(interp\ invokehidden\ -namespace) 1
+        set option(switch\ -matchvar) n
+        set option(switch\ -indexvar) n
     }
 
     # Things added in 8.6
     if {[info commands try] ne ""} {
         # Changed commands
-        set syntax(catch)        "c n? n?" ;# FIXA make a test for this
         set syntax(close)           "x x?"
         set syntax(chan\ close)     "x x?"
         set syntax(dict\ filter)    "x x x*"
@@ -459,7 +461,7 @@ proc buildDb {ch} {
         set syntax(chan\ pop)                "x"
         set syntax(chan\ push)               "x c"
         set syntax(dict\ map)                "nl x c"
-        set syntax(file\ tempfile)           "n? x?*"
+        set syntax(file\ tempfile)           "n? x?"
         set syntax(info\ coroutine)          0
         set syntax(interp\ cancel)           "o* x? x?"
         # New commands
@@ -468,13 +470,18 @@ proc buildDb {ch} {
         set syntax(try)          "c x*"
         set special(try) 1
         set syntax(throw)        "2"
+        # TODO: Autimatically recognise the command generated by coroutine
+        # This could be done using "dc", but that would warn in the common
+        # case of non-constant coro names.
         set syntax(coroutine)    "x x x*"
         set special(tailcall) 1
         set special(next) 1
         set syntax(yield)        "x?"
         set syntax(yieldto)        "x x*"
+        # lmap is handled specially, like foreach, but the presence of the
+        # syntax string tells the internals that it is allowed
         set syntax(lmap) "n x c"
-        #set special(lmap) 1 FIXA
+        set special(lmap) 1
         set syntax(zlib)             "s x*"
         set syntax(zlib\ adler32)    "x x?"
         set syntax(zlib\ compress)   "x x?"
@@ -499,12 +506,14 @@ proc buildDb {ch} {
         set syntax(tcl::prefix\ match)  "o* x x"
         set option(tcl::prefix\ match\ -message) x
         set option(tcl::prefix\ match\ -error) x
+
         set syntax(oo::class)    "s x*"
         set syntax(oo::class\ create) "do=_stdclass_oo cn?"
         set syntax(oo::class\ create::constructor) dk ;# Define constructor
         set syntax(oo::class\ create::superclass)  di ;# Define inheritance
         set syntax(oo::class\ create::method) "dm"    ;# Define method 
-        set syntax(oo::class\ create::destructor) c
+        set syntax(oo::class\ create::destructor) dd  ;# Define destructor
+        set syntax(oo::class\ create::variable) div*
         set syntax(oo::class\ create::export) x
         set syntax(oo::class\ create::class) x
         set syntax(oo::class\ create::deletemethod) "x x*"
@@ -517,37 +526,33 @@ proc buildDb {ch} {
         set syntax(oo::class\ create::unexport) "x x*"
 
         set syntax(_stdclass_oo) "s x*"
-        set subCmd(_stdclass_oo) "create new destroy variable"
+        set subCmd(_stdclass_oo) "create new destroy variable varname"
         set syntax(_stdclass_oo\ create) "dc=_obj,_stdclass_oo x?"
         set return(_stdclass_oo\ create) _obj,_stdclass_oo
         set syntax(_stdclass_oo\ new) 0
         set return(_stdclass_oo\ new) _obj,_stdclass_oo
         set syntax(_stdclass_oo\ destroy) 0
         set syntax(_stdclass_oo\ variable) n*
+        set syntax(_stdclass_oo\ varname) v
+        set return(_stdclass_oo\ varname) varName
         set syntax(info\ object) "s x x*"
         set syntax(info\ class)  "s x x*"
         set syntax(oo::copy)     "x x?"
         lappend ::kP TclOO
 
-        # FIXA: all oo::define
-        set syntax(oo::define)   "2: x cn : x s x x*"
-        set syntax(oo::objdefine)   "2: x cn : x s x x*"
-        set syntax(oo::define::method) "x cv"
-        set syntax(oo::objdefine::method) "x cv"
-        set syntax(oo::define\ method) "x cv"
-        set syntax(oo::objdefine\ method) "x cv"
-        set syntax(oo::define::constructor) "cv"
-        set syntax(oo::objdefine::constructor) "cv"
-        set syntax(oo::define\ constructor) "cv"
-        set syntax(oo::objdefine\ constructor) "cv"
-        set syntax(oo::define::denstructor) "cl"
-        set syntax(oo::objdefine::destructor) "cl"
-        set syntax(oo::define\ destructor) "cl"
-        set syntax(oo::objdefine\ destructor) "cl"
-        set syntax(oo::define::forward) "x x x*"
-        set syntax(oo::objdefine::forward) "x x x*"
-        set syntax(oo::define::unexport) "x x*"
-        set syntax(oo::objdefine::unexport) "x x*"
+        # oo::define has two formats, one for a "configure script" and one
+        # for single subcommands. Thus each command needs to be defined
+        # both as a command in the namespace, and as a subcommand.
+        set syntax(oo::define)      "2: do cn : do s x x*"
+        set syntax(oo::objdefine)   "2: do cn : do s x x*"
+        # Copy from class create
+        foreach item [array names syntax oo::class\ create::*] {
+            set tail [namespace tail $item]
+            set syntax(oo::define::$tail) $syntax($item)
+            set syntax(oo::define\ $tail) $syntax($item)
+            set syntax(oo::objdefine::$tail) $syntax($item)
+            set syntax(oo::objdefine\ $tail) $syntax($item)
+        }
 
         set syntax(oo::object)   "s x*" ;# FIXA?
         # Set up basic checking of self/my
@@ -783,6 +788,10 @@ proc buildDb {ch} {
             set option(chan\ puts) [list -nonewline]
         }            
     }
+    # Send cannot return info in some versions
+    if {![info exists option(send)]} {
+        set option(send) "-- -async -displayof"
+    }
 
     # The default for options is not to take a value unless 'p' is
     # used in the syntax definition.
@@ -832,7 +841,7 @@ proc buildDb {ch} {
     puts $ch [list lappend ::dbInfo $dbstring]
     puts $ch [list set ::dbTclVersion $::tcl_version]
     puts $ch [list set ::knownGlobals [lsort $::kG]]
-    puts $ch [list set ::knownCommands [lsort $::kC]]
+    puts $ch [list set ::knownCommands \n[join [lsort $::kC] \n]\n]
     puts $ch [list set ::knownPackages [lsort $::kP]]
 
     foreach a {syntax return subCmd option} {
